package ch.uzh.ifi.ddis.ifp.esper.cassandra;

/*
 * #%L
 * Cassandra for Esper
 * %%
 * Copyright (C) 2013 University of Zurich
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * #L%
 */

import com.espertech.esper.client.hook.VirtualDataWindowContext;
import com.espertech.esper.client.hook.VirtualDataWindowLookupContext;
import com.espertech.esper.client.hook.VirtualDataWindowLookupFieldDesc;

/**
 * <p>
 * Helper class for building a CQL query string from context information.
 * </p>
 * 
 * <p>
 * The projection part (i.e., the fields for the SELECT) are taken from
 * {@link com.espertech.esper.client.EventType#getPropertyNames()}. The event
 * type is taken from {@link VirtualDataWindowContext#getEventType()}.
 * </p>
 * 
 * <p>
 * If the {@link VirtualDataWindowLookupContext} provides hash fields, then
 * these are added as parameters to the WHERE clause.
 * </p>
 * 
 * @author Thomas Scharrenbach
 * @version 0.2.3
 * @since 0.2.0
 * 
 */
public class QueryStringBuilder {

	public static synchronized String createQueryString(
			VirtualDataWindowContext context,
			VirtualDataWindowLookupContext lookupContext, String table) {
		final StringBuffer queryBuffer = new StringBuffer();

		final String[] propertyNames = context.getEventType()
				.getPropertyNames();

		boolean firstColumn = true;
		queryBuffer.append("SELECT ");
		for (String propertyName : propertyNames) {
			if (firstColumn) {
				firstColumn = false;
			} else {
				queryBuffer.append(", ");
			}
			queryBuffer.append(String.format("%s ", propertyName));
		}

		queryBuffer.append(String.format("FROM %s ", table));

		// Only add where clause if hints for hash or btree lookups are
		// provided.
		if (!lookupContext.getHashFields().isEmpty()) {

			queryBuffer.append("WHERE ");

			boolean firstStatement = true;

			for (VirtualDataWindowLookupFieldDesc field : lookupContext
					.getHashFields()) {
				if (firstStatement) {
					firstStatement = false;
				} else {
					queryBuffer.append(" AND ");
				}
				queryBuffer.append(String.format(" %s %s ?",
						field.getPropertyName(), field.getOperator().getOp()));
			}
		}
		return queryBuffer.toString();
	}

}
